#!/bin/sh
# $Id: gateway.sh,v 1.0.9.1 1998/06/01 19:34:40 jens Exp $
# $Date: 1998/06/01 19:34:40 $
#
# File: gateway.sh
# Desc: definition of the GATEWAY  filtering function (ipgw)
#       definition of the TRUSTED  filtering function (tipgw)
#       definition of the FIREWALL filtering function (ipfw)
#       definition of the EXTERNAL filtering function (eipgw)
#
# Author : Jens Hellmerichs-Friedrich
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2, or (at your option)
#   any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
# (c) Copyright 1998 Jens Hellmerichs-Friedrich
#
# $Log: gateway.sh,v $
# Revision 1.0.9.1  1998/06/01 19:34:40  jens
# initial ci -r 1.0.9
#
# Revision 1.0  1998/06/01 19:32:24  jens
# initial ci -r 1.0
#
#

if [ -z "$GATEWAY" ]; then
GATEWAY=defined

# where is the firewall-package located ?
[ -z "$FWDIR" ] && FWDIR=`echo $0 | sed -e 's/[\/]etc[\/][A-Za-z0-9_\.-]*$//'`

# default
[ -z "$ip_cmd" ] && ip_cmd="ipfwadm"

case "$ip_cmd" in
	ipfwadm)
		IPFWADM=/sbin/ipfwadm
		IPFW="f_ipfwadm"
		IPFWCMD=`basename $IPFWADM`
                . $FWDIR/etc/ipfwadm.sh
		;;
	ipchains)
		IPFWADM=/sbin/ipchains
		IPFW="f_ipchains"
		IPFWCMD=`basename $IPFWADM`
                . $FWDIR/etc/ipchains.sh
		;;
	ipF)
		IPFWADM=/sbin/ipf
		IPFW="f_ipfilter"
		IPFWCMD=`basename $IPFWADM`
                . $FWDIR/etc/ipFilter.sh
		;;
esac

setup_file="not_defined"

# ----------------
# GATEWAY Function
# ----------------
# implementation of filtering rules for a gateway
# the gateway has an incoming and an outgoing device

# ipgw (
#       opt               add    | del
#       action            accept | deny
#       direction         in     | out
#       src               $ANY   | $INTERNAL | $INSRV | <ip-addr>
#       dest              $ANY   | $INTERNAL | $INSRV | <ip-addr>
#       src-prt           <port> | <from:to>
#       dest-prt          <port> | <from:to>
#       proto             tcp    | udp       | icmp
#       ack               ""     | ack
#       log               log
#      )

ipgw ()
{
  # interprete parameters
  
  opt="-a"
  [ "$1" = del ] && opt="-d"
  shift

  ack=""
  [ "-$8" = "-ack" ] && ack="-k"

  iplog=""
  [ "$8" = "log" ] && iplog="log"

  srcprt="$5"
  [ "$5" = "any" ] && srcprt=""

  destprt="$6"
  [ "$6" = "any" ] && destprt=""

  proto="$7"
  [ "$7" = "any" ] && proto=""

  # there are 3 incoming and 3 outgoing routings:  $OUT_DEV  -> $IN_DEV
  # I: $ANY      -> $INTERNAL    $OUT_DEV0 -> $IN_DEV0
  # I: $ANY      -> $INSRV       $OUT_DEV0 -> $IN_DEV1
  # I: $INSRV    -> $INTERNAL    $IN_DEV1  -> $IN_DEV0
  # O: $INTERNAL -> $ANY         $IN_DEV0  -> $OUT_DEV0
  # O: $INTERNAL -> $INSRV       $IN_DEV0  -> $IN_DEV1
  # O: $INSRV    -> $ANY         $IN_DEV1  -> $OUT_DEV0

  if [ $# -gt 3 ]; then
    case $2 in
      -I) if [ `echo $PERIMETER | grep "$3" | wc -l` -gt 0 ]; then
              OUT_DEV=$IN_DEV1
          else
              OUT_DEV=$OUT_DEV0
          fi
          if [ `echo $PERIMETER | grep "$4" | wc -l` -gt 0 ]; then
              IN_DEV=$IN_DEV1
          else
              IN_DEV=$IN_DEV0
          fi
          if [ "x$OUT_DEV" != x -a "x$IN_DEV" != x ]; then
            for dev in $OUT_DEV; do
                $IPFW -I "$opt" "$1" "$ack" "$3" "$srcprt" "$4" "$destprt" "$proto" "$dev" "$iplog"
            done
            if [ "$proto" != tcp -o -z "$iplog" ]; then
              for dev in $IN_DEV; do
                $IPFW -O "$opt" "$1" "$ack" "$3" "$srcprt" "$4" "$destprt" "$proto" "$dev"
              done
              $IPFW -F "$opt" "$1" "$ack" "$3" "$srcprt" "$4" "$destprt" "$proto" -b
            fi
          else
            if [ "x$4" = "x$FW" -a "x$OUT_DEV" != x ]; then
              for dev in $OUT_DEV; do
                  $IPFW -I "$opt" "$1" "$ack" "$3" "$srcprt" "$4" "$destprt" "$proto" "$dev" "$iplog"
              done
            fi
          fi
          ;;

      -O) if [ `echo $PERIMETER | grep "$4" | wc -l` -gt 0 ]; then
              OUT_DEV=$IN_DEV1
          else
              OUT_DEV=$OUT_DEV0
          fi
          if [ `echo $PERIMETER | grep "$3" | wc -l` -gt 0 ]; then
              IN_DEV=$IN_DEV1
          else
              IN_DEV=$IN_DEV0
          fi

          if [ "x$OUT_DEV" != x -a "x$IN_DEV" != x ]; then
            for dev in $IN_DEV; do
                $IPFW -I "$opt" "$1" "$ack" "$3" "$srcprt" "$4" "$destprt" "$proto" "$dev" "$iplog"
            done
            if [ "$proto" != tcp -o -z "$iplog" ]; then
              for dev in $OUT_DEV; do
                $IPFW -O "$opt" "$1" "$ack" "$3" "$srcprt" "$4" "$destprt" "$proto" "$dev"
              done
              $IPFW -F "$opt" "$1" "$ack" "$3" "$srcprt" "$4" "$destprt" "$proto" -b
            fi
          else
            if [ "x$3" = "x$FW" -a "x$IN_DEV" != x ]; then
              for dev in $OUT_DEV; do
                  $IPFW -O "$opt" "$1" "$ack" "$3" "$srcprt" "$4" "$destprt" "$proto" "$dev" "$iplog"
              done
            fi
          fi
          ;;
    esac
  fi
}


# -----------------
# FIREWALL Function
# -----------------
# implementation of filtering rules for the firewall itself
# the firewall has only an incommnig device
# (if we use an outgoing device too, it would be a gateway)

# ipfw (
#       opt               add    | del
#       action            accept | deny
#       direction         in     | out
#       src               $ANY   | $INTERNAL | $INSRV | <ip-addr>
#       dest              $ANY   | $INTERNAL | $INSRV | <ip-addr>
#       src-prt           <port> | <from:to>
#       dest-prt          <port> | <from:to>
#       proto             tcp    | udp       | icmp
#       ack               ""     | ack
#      )

ipfw ()
{
  opt="-a"
  [ "$1" = del ] && opt="-d"
  [ "$1" = ins ] && opt="-i"
  shift

  ack=""
  [ "-$8" = "-ack" ] && ack="-k"

  iplog=""
  [ "$8" = "log" ] && iplog="log"

  srcprt="$5"
  [ "$5" = "any" ] && srcprt=""

  destprt="$6"
  [ "$6" = "any" ] && destprt=""

  proto="$7"
  [ "$7" = "any" ] && proto=""

  if [ $# -gt 3 ]; then
    case $2 in
      -I) 
          for i in 1; do
          if [ `echo "$3" | grep -c ^$DOMAIN` -gt 0 ]; then
            if=$IN_DEV0
            break;
          fi
          if [ `echo "$3" | grep -c ^$FW\$` -gt 0 ]; then
            if=
            #if=lo
            break;
          fi
          if [ `echo "$3" | grep -c ^$INSRV\$` -gt 0 ]; then
            if=$IN_DEV1
            break;
          fi
          if [ `echo "$3" | grep -c ^$GW\$` -gt 0 ]; then
            if=$IN_DEV1
            break;
          fi
          if [ `echo "$3" | grep -c $DOMAIN` -gt 0 ]; then
            if=$IN_DEV0
            break;
          fi
          if [ `echo "$3" | grep -c ^$ANY\$` -gt 0 ]; then
            if=$OUT_DEV0
            break;
          fi
          if [ `echo " $PERIMETER" | grep -c "$3"` -gt 0 ]; then
                   if=$IN_DEV1
          else
                   if=$OUT_DEV0
          fi
          done
          if [ "x$if" != "x" ]; then
            for dev in $if; do
                $IPFW -I "$opt" "$1" "$ack" "$3" "$srcprt" "$4" "$destprt" "$proto" "$dev" "$iplog"
               #$IPFW -F "$opt" "$1" "$ack" "$3" "$srcprt" "$4" "$destprt" "$proto" -b
            done
          fi
          ;;

      -O) 
          for i in 1; do
          if [ `echo "$4" | grep -c ^$DOMAIN` -gt 0 ]; then
            if=$IN_DEV0
            break;
          fi
          if [ `echo "$4" | grep -c ^$FW\$` -gt 0 ]; then
            if=
            #if=lo
            break;
          fi
          if [ `echo "$4" | grep -c ^$INSRV\$` -gt 0 ]; then
            if=$IN_DEV1
            break;
          fi
          if [ `echo "$4" | grep -c ^$GW\$` -gt 0 ]; then
            if=$IN_DEV1
            break;
          fi
          if [ `echo "$4" | grep -c $DOMAIN` -gt 0 ]; then
            if=$IN_DEV0
            break;
          fi
          if [ `echo "$4" | grep -c ^$ANY\$` -gt 0 ]; then
            if=$OUT_DEV0
            break;
          fi
          if [ `echo " $PERIMETER" | grep -c "$3"` -gt 0 ]; then
                   if=$IN_DEV1
          else
                   if=$OUT_DEV0
          fi
          done
          if [ "x$if" != "x" ]; then
            for dev in $if; do
                $IPFW -O "$opt" "$1" "$ack" "$3" "$srcprt" "$4" "$destprt" "$proto" "$dev" "$iplog"
               #$IPFW -F "$opt" "$1" "$ack" "$3" "$srcprt" "$4" "$destprt" "$proto" -b
            done
          fi
          ;;
    esac
  fi
}

# ----------------------
# TRUSTED HOSTS Function 
# ----------------------
# implementation of filtering rules for "Trusted Hosts"
# we have an incoming device and an outgoing device
# -> use the firewall as a gateway
# be carefull using this form of the gateway, because
# every connection is allowed for "trusted hosts"

# tipgw (
#        opt            add | del | ins
#        host           <host>
#        device         <device>
#       )

tipgw ()
{
  opt="-a"
  [ "$1" = del ] && opt="-d"
  [ "$1" = ins ] && opt="-i"
  shift

  iplog=""
  [ "$3" = "log" ] && iplog="log"


  if [ $# -gt 1 ]; then
    if [ "x$IN_DEV0" != x ]; then
      $IPFW -I "$opt" accept "" "$1"        "" "$INTERNAL" "" "" "$2"       "$iplog"
      $IPFW -O "$opt" accept "" "$1"        "" "$INTERNAL" "" "" "$IN_DEV0"
      $IPFW -F "$opt" accept "" "$1"        "" "$INTERNAL" "" "" -b
      $IPFW -I "$opt" accept "" "$INTERNAL" "" "$1"        "" "" "$IN_DEV0"
      $IPFW -O "$opt" accept "" "$INTERNAL" "" "$1"        "" "" "$2"       "$iplog"
      $IPFW -F "$opt" accept "" "$INTERNAL" "" "$1"        "" "" -b
    else
      $IPFW -I "$opt" accept "" "$1"        "" "$INTERNAL" "" "" "$2"
      $IPFW -O "$opt" accept "" "$INTERNAL" "" "$1"        "" "" "$2"
    fi
    if [ "x$IN_DEV1" != x -a "x$IN_DEV1" != "x$IN_DEV0" ]; then
      $IPFW -I "$opt" accept "" "$1"     "" "$INSRV" "" "" "$2"       "$iplog"
      $IPFW -O "$opt" accept "" "$1"     "" "$INSRV" "" "" "$IN_DEV1"
      $IPFW -F "$opt" accept "" "$1"     "" "$INSRV" "" "" -b
      $IPFW -I "$opt" accept "" "$INSRV" "" "$1"     "" "" "$IN_DEV1"
      $IPFW -O "$opt" accept "" "$INSRV" "" "$1"     "" "" "$2"       "$iplog"
      $IPFW -F "$opt" accept "" "$INSRV" "" "$1"     "" "" -b
    fi
  fi
}


# -------------------------------------
# External Gateway with specific device
# -------------------------------------
# implementation of filtering rules for an "External Gateway Interface"
# the external device is specified, the internal device is $IN_DEV0

# eipgw (
#       opt               add    | del | ins
#       action            accept | deny
#       direction         in     | out
#       src               $ANY   | $INTERNAL | $INSRV | <ip-addr>
#       dest              $ANY   | $INTERNAL | $INSRV | <ip-addr>
#       src-prt           <port> | <from:to>
#       dest-prt          <port> | <from:to>
#       proto             tcp    | udp       | icmp
#       ack               ""     | ack
#       dev               (e.g. ippp1)
#      )

eipgw ()
{
  opt="-a"
  [ "$1" = del ] && opt="-d"
  [ "$1" = ins ] && opt="-i"
  shift

  ack=""
  [ "-$8" = "-ack" ] && ack='-k'

  srcprt="$5"
  [ "$5" = "any" ] && srcprt=""

  destprt="$6"
  [ "$6" = "any" ] && destprt=""

  proto="$7"
  [ "$7" = "any" ] && proto=""

  OUT_DEV=$9
  IN_DEV=$IN_DEV0

  case $2 in
      -I)
          $IPFW -I "$opt" "$1" "$ack" "$3" "$srcprt" "$4" "$destprt" "$proto" "$OUT_DEV"
          $IPFW -O "$opt" "$1" "$ack" "$3" "$srcprt" "$4" "$destprt" "$proto" "$IN_DEV"
          $IPFW -F "$opt" "$1" "$ack" "$3" "$srcprt" "$4" "$destprt" "$proto" -b
          ;;

      -O)
          $IPFW -I "$opt" "$1" "$ack" "$3" "$srcprt" "$4" "$destprt" "$proto" "$IN_DEV"
          $IPFW -O "$opt" "$1" "$ack" "$3" "$srcprt" "$4" "$destprt" "$proto" "$OUT_DEV"
          $IPFW -F "$opt" "$1" "$ack" "$3" "$srcprt" "$4" "$destprt" "$proto" -b
          ;;
  esac
}

fi

